#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
NOTE: Do not use this module to add random utils.
We want to avoid having a generic utils.py dumping ground.
"""

from debtcollector import moves

from neutron_lib.api import attributes

populate_project_info = moves.moved_function(
    attributes.populate_project_info, 'populate_project_info', __name__)
